﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Net;
using System.Security.Cryptography.X509Certificates;
#endregion

namespace Cnas.Siui.TestConnectivity.Services.Abstracts
{
    /// <summary>
    /// Abstract Service Class that invokes web service methods. Before calling, a request is made for the OCSP server token that will be attached to the original request  
    /// </summary>
    /// <typeparam name="TOutput"></typeparam>
    public abstract class AbstractOnlineRemoteService<TOutput> : AbstractRemoteService<TOutput>
    {
        /// <summary>
        /// Instance of the OCSPRemoteService
        /// </summary>
        private readonly CheckOcspRemoteService ocspRemoteService = null;
        private bool keepOcspToken;

        private string ocspToken = null;
        protected string OcspToken
        {
            get
            {
                lock( this )
                {
                    if( string.IsNullOrEmpty( this.ocspToken ) || keepOcspToken==false )
                    {
                        //refresh ocsp token
                        this.ocspToken = this.ocspRemoteService.GetResponse();
                    }
                    return this.ocspToken;
                }
            } 
            set { this.ocspToken = value; } }

        public string Username { get; private set; }
        public string Password { get; private set; }

        /// <summary>
        /// Creates an instance of class <see cref="AbstractOnlineRemoteService"/>
        /// </summary>
        protected AbstractOnlineRemoteService( string url, string ocspUrl, string username, string password, X509Certificate2 certificate, int requestTimeout, bool keepOcspToken = true )
            : base( url, certificate, requestTimeout )
        {
            this.Username = username;
            this.Password = password;
            //Initializing the OCSPRemoteService
            this.ocspRemoteService = new CheckOcspRemoteService( ocspUrl, this.Username, this.Certificate, this.RequestTimeout );
            this.keepOcspToken = keepOcspToken;
        }

        public override TOutput GetResponse()
        {
            try
            {
                return base.GetResponse();
            }
            catch( WebException )
            {
                //Force OcspToken to be refresh next time it's used
                this.OcspToken = null;
                return base.GetResponse();
            }
        }

        /// <summary>
        /// Creates the requests and attaches the OCSP server token
        /// </summary>
        protected override HttpWebRequest CreateRequest( Uri uri )
        {
            var request = base.CreateRequest( uri );
            request.Credentials = this.GetCredentials( new Uri( this.WebServiceUrl ) );
            var ocspServerToken = this.OcspToken;
            if( ocspServerToken != null )
            {
                //adding the OCSP Server token
                request.Headers.Add( "OSCP_RESPONSE", ocspServerToken );
            }
            request.ContentType = "text/xml";
            return request;
        }

        /// <summary>
        /// Retrieves credential for given uri
        /// </summary>
        private ICredentials GetCredentials( Uri uri )
        {
            // create credential cache
            return new CredentialCache
                       {
                           {uri, "Basic", new NetworkCredential( this.Username, this.Password )},
                           {uri, "Digest", new NetworkCredential( this.Username, this.Password )}
                       };
        }
    }
}