﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Services.Protocols;
using Cnas.Siui.WebServices;
using Cnas.Siui.WebServices.Utils;
#endregion

namespace Cnas.Siui.TestConnectivity.Services.Abstracts
{
    /// <summary>
    /// Service that should be use to invoke the web service method
    /// </summary>
    /// <typeparam name="TWebService">WebService Type</typeparam>
    /// <typeparam name="TWebserviceOutput">The type that the web service method returns</typeparam>
    /// <typeparam name="TInput">Input type parameters used for invoking the web service method</typeparam>
    /// <typeparam name="TOutput">The output type that this service should retrieve after processing the web response</typeparam>
    public abstract class AbstractWebService<TWebService, TWebserviceOutput, TOutput, TInput>
        where TWebService: SoapHttpClientProtocol, IOnlineService, new()
    {
        public string Url { get; private set; }
        public string Username { get; private set;}
        public string Password { get; private set;}
        public X509Certificate2 Certificate { get; private set;}
        public int RequestTimeout { get; private set;}
        public string OCSPUrl;
        private bool keepOcspToken;
        private CheckOcspRemoteService ocspRemoteService = null;
        private string ocspToken = null;
        private string OcspToken
        {
            get
            {
                if( string.IsNullOrEmpty( this.ocspToken ) || this.keepOcspToken == false )
                {
                    this.ocspToken = this.ocspRemoteService.GetResponse();
                }
                return this.ocspToken;
            }
            set
            {
                lock( this )
                {
                    this.ocspToken = value;
                }
            }
        }


        /// <summary>
        /// Creates an instance of class <see cref="AbstractWebService"/>
        /// </summary>
        protected AbstractWebService( string url, string username, string password, X509Certificate2 certificate, string ocspUrl, int requestTimeout, bool keepOcspToken = true )
        {
            this.Url = url;
            this.Username = username;
            this.Password = password;
            this.Certificate = certificate;
            this.RequestTimeout = requestTimeout;
            this.keepOcspToken = keepOcspToken;
            this.OCSPUrl = ocspUrl;
            this.ocspRemoteService = new CheckOcspRemoteService( this.OCSPUrl, this.Username, this.Certificate, this.RequestTimeout );
        }

        /// <summary>
        /// Invokes the webservice
        /// </summary>
        protected abstract TWebserviceOutput CallWebServiceMethod( TWebService webService, TInput parameters );

        #region public methods 
        /// <summary>
        /// Get Response
        /// </summary>
        /// <param name="parameters"></param>
        /// <returns></returns>
        public TOutput GetResponse( TInput parameters )
        {
            TWebserviceOutput response;
            try
            {
                response = this.GetWebServiceResponse( parameters );
            }
            catch( WebException )
            {
                //refresh token and retry webservice call
                this.OcspToken = null;
                response = this.GetWebServiceResponse( parameters );
                throw;
            }
            return this.ProcessResponse( parameters, response );
        }

        /// <summary>
        /// Invokes the webservice method asynchronously
        /// </summary>
        public bool SendRequest( TInput parameters, Action<TInput, TOutput> onCompleted, Action<TInput, Exception> onFaulted = null )
        {
            try
            {
                this.SendRequestAsync( parameters, onCompleted, onFaulted );
                return true;
            }
            catch( AggregateException )
            {
                return false;
            }
        }
        #endregion

        /// <summary>
        /// Invokes the webservice method asynchronously
        /// </summary>
        protected virtual Task<TOutput> SendRequestAsync( TInput parameters, Action<TInput, TOutput> onCompleted, Action<TInput, Exception> onFaulted )
        {
            try
            {
                return Task<TOutput>.Factory
                    .StartNew( () => this.GetResponse( parameters ))
                    .ContinueWith( prev =>
                                   {
                                       if( prev.IsFaulted )
                                       {
                                           if( onFaulted != null )
                                           {
                                               onFaulted( parameters, prev.Exception );
                                           }
                                       }
                                       else if( prev.IsCompleted && prev.IsCanceled == false )
                                       {
                                           if( onCompleted != null )
                                           {
                                               onCompleted( parameters, prev.Result );
                                           }
                                           return prev.Result;
                                       }

                                       return default( TOutput );
                                   },
                                   CancellationToken.None,
                                   TaskContinuationOptions.ExecuteSynchronously,
                                   TaskScheduler.FromCurrentSynchronizationContext()
                    );
            }
            catch( AggregateException ex )
            {
                if( onFaulted != null )
                {
                    onFaulted( parameters, ex );
                }
            }

            return null;
        }

        /// <summary>
        /// Calls the webservice methods. Invokes the OCSPRemoteService for the service token, attaches it the ongoing requst
        /// </summary>
        protected virtual TWebserviceOutput GetWebServiceResponse( TInput parameters )
        {
            Thread.CurrentThread.Priority = ThreadPriority.BelowNormal;
            CookieJar.Clear();
            var webService = new TWebService();

            var wsUri = new Uri( this.Url );
            webService.Url = wsUri.AbsoluteUri;
            webService.Credentials = new CredentialCache
                                         {
                                             {wsUri, "Basic", new NetworkCredential( this.Username, this.Password )},
                                             {wsUri, "Digest", new NetworkCredential( this.Username, this.Password )}
                                         };
            webService.SetupWebRequest = request =>

                                             {
                                                 var httpRequest = (HttpWebRequest) request;
                                                 if( this.Certificate != null )
                                                 {
                                                     httpRequest.ClientCertificates.Add(this.Certificate);
                                                 }
                                                 var ocspToken = this.OcspToken;
                                                 if( ocspToken != null )
                                                 {
                                                     httpRequest.Headers.Add( "OSCP_RESPONSE", ocspToken );
                                                 }
                                                 return httpRequest;
                                             };
            // issue call to web method (overrided in actual classes)
            return this.CallWebServiceMethod( webService, parameters );
        }

        /// <summary>
        /// Processes the web response
        /// </summary>
        protected abstract TOutput ProcessResponse( TInput parameters, TWebserviceOutput response );
    }
}