﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
#endregion

namespace Cnas.Siui.TestConnectivity.Utils
{
    /// <summary>
    /// BigInteger is a general-purpose unbounded integer implementation consistent with C# and .NET numeric type conventions
    /// </summary>
    [Serializable] //add [NonSerialized] attribute to any (possible future) fields other than digits and isneg
    [ComVisible( false )]
    [StructLayout( LayoutKind.Auto )] //since we don't care about COM interop
    public struct BigInteger : IComparable<BigInteger>, IEquatable<BigInteger>, IConvertible, IXmlSerializable
    {
#pragma warning disable 3001, 3002

        #region Instance Fields
        private LinkedList<byte> digits; //is null if and only if bigint is zero
        private bool isneg; //value irrelevant if digits is null
        #endregion Instance Fields

        #region Constructors
        //as is more natural for ValueTypes, we use explicit and and implicit conversion operators 
        //(and Parse for strings) in-lue of equivalent contructors (and to spare redundancy)

        /// <summary>
        /// shallow copy
        /// </summary>
        private BigInteger( LinkedList<byte> digits, bool isneg )
        {
            this.digits = digits;
            this.isneg = isneg;
        }
        #endregion Constructors

        #region Static Fields
        //being common and useful privately, we deduce the same for the public

        public static readonly BigInteger Zero = new BigInteger(); //data null, isneg false
        public static readonly BigInteger One = BigInteger.Parse( "1" );
        public static readonly BigInteger NegativeOne = BigInteger.Parse( "-1" );
        public static readonly BigInteger Two = BigInteger.Parse( "2" );
        #endregion Static Fields

        #region Instance Properties
        /// <summary>
        /// Traverse the unsigned digits of this BigInteger, left to right.
        /// </summary>
        public IEnumerable<byte> DigitsLeftToRight //DigitsL2R so tempting!
        {
            get
            {
                if( digits == null )
                {
                    yield return (byte)0;
                }
                else
                {
                    for( LinkedListNode<byte> cur = digits.First; cur != null; cur = cur.Next )
                    {
                        yield return cur.Value;
                    }
                }
            }
        }

        /// <summary>
        /// Traverse the unsigned digits of this BigInteger, right to left.
        /// </summary>
        public IEnumerable<byte> DigitsRightToLeft //DigitsR2L
        {
            get
            {
                if( digits == null )
                {
                    yield return (byte)0;
                }
                else
                {
                    for( LinkedListNode<byte> cur = digits.Last; cur != null; cur = cur.Previous )
                    {
                        yield return cur.Value;
                    }
                }
            }
        }

        /// <summary>
        /// Iterate the proper divisors of this BigInteger.  Zero yeilds no divisors.  Negatives yeild negative divisors.
        /// </summary>
        public IEnumerable<BigInteger> ProperDivisors //naive implementation
        {
            get
            {
                if( this > BigInteger.Zero )
                {
                    BigInteger half = BigInteger.DivideCeiling( this, BigInteger.Two );
                    BigInteger cur = BigInteger.One;
                    do
                    {
                        if( this % cur == BigInteger.Zero )
                        {
                            yield return cur;
                        }

                        cur++;
                    } while( cur <= half );
                }
                else if( this < BigInteger.Zero )
                {
                    BigInteger half = BigInteger.DivideFloor( this, BigInteger.Two );
                    BigInteger cur = BigInteger.NegativeOne;
                    do
                    {
                        if( this % cur == BigInteger.Zero )
                        {
                            yield return cur;
                        }

                        cur--;
                    } while( cur >= half );
                }
                //else //is zero - NOT PRACTICAL
                //{
                //    BigInteger cur = BigInteger.Zero;
                //    while (true)
                //        yield return ++cur;
                //}
            }
        }

        /// <summary>
        /// Iterate the divisors of this BigInteger.  Zero yeilds no divisors.  Negatives yeild negative divisors.
        /// </summary>
        public IEnumerable<BigInteger> Divisors //naive implementation
        {
            get
            {
                foreach( BigInteger cur in ProperDivisors )
                {
                    yield return cur;
                }

                if( this.digits != null ) //not zero
                {
                    yield return this;
                }
            }
        }
        #endregion Instance Properties

        #region Equals, GetHashCode, and ToString Overrides
        /// <summary>
        /// Determines whether this BigInteger is equivalent to the given object
        /// </summary>
        public override bool Equals( object obj )
        {
            if( object.ReferenceEquals( obj, null ) )
            {
                return false;
            }

            return ( obj is BigInteger ) && this.Equals( (BigInteger)obj );
        }

        /// <summary>
        /// Returns the hashcode for this BigInteger
        /// </summary>
        public override int GetHashCode()
        {
            if( digits == null )
            {
                return 0;
            }

            int count = this.digits.Count;
            int code = count;
            unchecked
            {
                foreach( byte b in digits )
                {
                    code = code * ( b == 0 ? count : b );
                    if( code == 0 )
                    {
                        code = count;
                    }
                }
            }

            return ( isneg ? -1 : 1 ) * code;
        }

        /// <summary>
        /// String representation of this BigInteger
        /// </summary>
        public override string ToString()
        {
            if( digits == null )
            {
                return "0";
            }

            var sb = new System.Text.StringBuilder();

            if( isneg )
            {
                sb.Append( "-" );
            }

            foreach( byte digit in this.digits )
            {
                sb.Append( digit.ToString() );
            }

            return sb.ToString();
        }
        #endregion Equals, GetHashCode, and ToString Overrides

        #region Comparison and Equality
        /// <summary>
        /// Compare two BigInts
        /// </summary>
        public static int Compare( BigInteger lhs, BigInteger rhs )
        {
            return lhs.CompareTo( rhs );
        }

        /// <summary>
        /// Compare this BigInteger to another
        /// </summary>
        public int CompareTo( BigInteger other ) //was less than
        {
            //permutations of this and rhs being zero
            if( this.digits == null && other.digits == null )
            {
                return 0; //they are equal (both 0)
            }
            else if( this.digits == null && other.digits != null )
            {
                return other.isneg ? 1 : -1;
            }
            else if( this.digits != null && other.digits == null )
            {
                return this.isneg ? -1 : 1;
            }

            //this is negative and rhs is positive
            if( this.isneg && !other.isneg )
            {
                return -1;
            }
            else if( !this.isneg && other.isneg )
            {
                return 1;
            }
            else if( this.isneg && other.isneg ) //this and rhs are negative
            {
                if( this.digits.Count > other.digits.Count )
                {
                    return -1;
                }
                else if( this.digits.Count < other.digits.Count )
                {
                    return 1;
                }
                else //count == count
                {
                    return CompareFirstDiffDigit( other.digits, this.digits );
                }
            }
            else //!this.isneg && !rhs.isneg (this and rhs are positive
            {
                if( this.digits.Count > other.digits.Count )
                {
                    return 1;
                }
                else if( this.digits.Count < other.digits.Count )
                {
                    return -1;
                }
                else //count == count
                {
                    return CompareFirstDiffDigit( this.digits, other.digits );
                }
            }
        }

        public static bool operator <( BigInteger lhs, BigInteger rhs )
        {
            return lhs.CompareTo( rhs ) == -1;
        }

        public static bool operator <=( BigInteger lhs, BigInteger rhs )
        {
            int result = lhs.CompareTo( rhs );
            return ( result == -1 || result == 0 );
        }

        public static bool operator >( BigInteger lhs, BigInteger rhs )
        {
            return lhs.CompareTo( rhs ) == 1;
        }

        public static bool operator >=( BigInteger lhs, BigInteger rhs )
        {
            int result = lhs.CompareTo( rhs );
            return ( result == 1 || result == 0 );
        }

        public static bool operator ==( BigInteger lhs, BigInteger rhs )
        {
            return lhs.CompareTo( rhs ) == 0;
        }

        public static bool operator !=( BigInteger lhs, BigInteger rhs )
        {
            return !( lhs == rhs );
        }

        /// <summary>
        /// arguments may not be null
        /// </summary>
        private static bool IsLessThan( LinkedList<byte> lhs_digits, LinkedList<byte> rhs_digits )
        {
            return ( lhs_digits.Count == rhs_digits.Count && ( CompareFirstDiffDigit( lhs_digits, rhs_digits ) == -1 ) ) || ( lhs_digits.Count < rhs_digits.Count );
        }

        /// <summary>
        /// compares first most signifigant digits
        /// assumes lhs and rhs are both non-null, with equal counts greater than 0
        /// </summary>
        private static int CompareFirstDiffDigit( LinkedList<byte> lhs, LinkedList<byte> rhs )
        {
            LinkedListNode<byte> cur_lhs = lhs.First;
            LinkedListNode<byte> cur_rhs = rhs.First;
            for( int i = 0; i < lhs.Count; i++, cur_lhs = cur_lhs.Next, cur_rhs = cur_rhs.Next )
            {
                if( cur_lhs.Value != cur_rhs.Value )
                {
                    return cur_lhs.Value < cur_rhs.Value ? -1 : 1;
                }
            }

            return 0; //all digits equal
        }

        /// <summary>
        /// Determines whether this BigInteger is equivalent to the given BigInteger
        /// </summary>
        public bool Equals( BigInteger other )
        {
            return this == other;
        }
        #endregion Comparison and Equality

        #region Arithmetic Algorithms and Operators
        public static BigInteger operator +( BigInteger lhs, BigInteger rhs )
        {
            return Add( lhs, rhs );
        }

        /// <summary>
        /// Add two BigInts
        /// </summary>
        public static BigInteger Add( BigInteger lhs, BigInteger rhs )
        {
            if( lhs.digits == null && rhs.digits == null ) //0 + 0 = 0
            {
                return BigInteger.Zero;
            }
            else if( lhs.digits == null && rhs.digits != null ) //0 + b = b
            {
                return rhs;
            }
            else if( lhs.digits != null && rhs.digits == null ) //a + b = 0
            {
                return lhs;
            }
            else
            {
                return AddPhase2( lhs.digits, lhs.isneg, rhs.digits, rhs.isneg );
            }
        }

        /// <summary>
        /// operands are non-zero
        /// 
        /// There are 4 general cases:
        /// 1) c = a + b
        /// 2) c = -a + -b = -(a + b)
        /// 3) c = a + -b = a - b
        /// 4) c = -a + b = b - a
        /// special: c = a + b = b + a
        /// </summary>
        private static BigInteger AddPhase2( LinkedList<byte> lhs_digits, bool lhs_isneg, LinkedList<byte> rhs_digits, bool rhs_isneg )
        {
            //use algerbra to make sure a and b are positive
            if( lhs_isneg && rhs_isneg ) // (-a) + (-b) = -(a + b)
            {
                return new BigInteger( AddPhase3( lhs_digits, rhs_digits ), true );
            }
            else if( !lhs_isneg && rhs_isneg ) // a + (-b) = a - b
            {
                return SubtractPhase3( lhs_digits, rhs_digits );
            }
            else if( lhs_isneg && !rhs_isneg ) // (-a) + b = b - a
            {
                return SubtractPhase3( rhs_digits, lhs_digits );
            }
            else //now a and b are positive
            {
                return new BigInteger( AddPhase3( lhs_digits, rhs_digits ), false );
            }
        }

        /// <summary>
        /// operands are non-zero and positive
        /// </summary>
        private static LinkedList<byte> AddPhase3( LinkedList<byte> lhs_digits, LinkedList<byte> rhs_digits )
        {
            //make sure a is greater than b
            if( IsLessThan( lhs_digits, rhs_digits ) )
            {
                return AddPhase4( rhs_digits, lhs_digits ); // a + b = b + a
            }
            else //now a and b are positive and a is greater than b
            {
                return AddPhase4( lhs_digits, rhs_digits );
            }
        }

        /// <summary>
        /// operands are non-zero, positive, and lhs >= rhs
        /// </summary>
        private static LinkedList<byte> AddPhase4( LinkedList<byte> lhs_digits, LinkedList<byte> rhs_digits )
        {
            LinkedList<byte> sum_digits = new LinkedList<byte>();
            LinkedListNode<byte> cur_lhs = lhs_digits.Last;
            LinkedListNode<byte> cur_rhs = rhs_digits.Last;
            int carry = 0;
            int rhs_topindex = rhs_digits.Count - 1;
            for( int i = 0;; )
            {
                byte val_lhs = cur_lhs.Value;
                byte val_rhs = i <= rhs_topindex ? cur_rhs.Value : (byte)0;
                int sum = ( val_lhs + val_rhs + carry );
                carry = 0;
                if( sum > 9 )
                {
                    sum -= 10;
                    carry = 1;
                }
                sum_digits.AddFirst( (byte)sum );

                if( cur_lhs == lhs_digits.First )
                {
                    break;
                }

                cur_lhs = cur_lhs.Previous;
                if( ++i <= rhs_topindex )
                {
                    cur_rhs = cur_rhs.Previous;
                }
            }
            if( carry == 1 )
            {
                sum_digits.AddFirst( (byte)carry );
            }

            return sum_digits;
        }

        /// <summary>
        /// operands are non-zero and both positive or both negative.  mutates lhs_digits.  because of extra checks required to handeld both a less than b
        /// and b less than a cases, slightly slower than AddPhase4, but when doing accumlative addition (such as Multiply), performance
        /// is much increased by sparing repeated large memory allocation for tempory states.
        /// </summary>
        private static void AddTo( LinkedList<byte> lhs_digits, LinkedList<byte> rhs_digits )
        {
            LinkedListNode<byte> cur_lhs = lhs_digits.Last;
            LinkedListNode<byte> cur_rhs = rhs_digits.Last;
            int carry = 0;
            int rhs_topindex = rhs_digits.Count - 1;
            int lhs_topindex = lhs_digits.Count - 1;
            int max_index = Math.Max( rhs_topindex, lhs_topindex );
            for( int i = 0;; )
            {
                byte val_lhs = i <= lhs_topindex ? cur_lhs.Value : (byte)0;
                byte val_rhs = i <= rhs_topindex ? cur_rhs.Value : (byte)0;
                int sum = ( val_lhs + val_rhs + carry );
                carry = 0;
                if( sum > 9 )
                {
                    sum -= 10;
                    carry = 1;
                }

                if( i > lhs_topindex )
                {
                    lhs_digits.AddFirst( (byte)sum );
                    if( i == max_index )
                    {
                        break;
                    }

                    cur_lhs = lhs_digits.First;
                }
                else
                {
                    cur_lhs.Value = (byte)sum;
                    if( i == max_index )
                    {
                        break;
                    }

                    cur_lhs = cur_lhs.Previous;
                }

                if( ++i <= rhs_topindex )
                {
                    cur_rhs = cur_rhs.Previous;
                }
            }

            if( carry == 1 )
            {
                lhs_digits.AddFirst( (byte)carry );
            }
        }

        public static BigInteger operator -( BigInteger lhs, BigInteger rhs )
        {
            return Subtract( lhs, rhs );
        }

        /// <summary>
        /// Subtract the right-hand-side from the left-hand-side
        /// </summary>
        public static BigInteger Subtract( BigInteger lhs, BigInteger rhs )
        {
            if( lhs.digits == null && rhs.digits == null ) //0 - 0 = 0,  (may consider value of implementing a - a = 0 [SEE Subtract3])
            {
                return BigInteger.Zero;
            }
            else if( lhs.digits == null && rhs.digits != null ) //0 - (a) = -a, 0 - (-a) = a
            {
                return new BigInteger( rhs.digits, !rhs.isneg );
            }
            else if( lhs.digits != null && rhs.digits == null ) //a - 0 = a
            {
                return lhs;
            }
            else
            {
                return SubtractPhase2( lhs.digits, lhs.isneg, rhs.digits, rhs.isneg );
            }
        }

        /// <summary>
        /// a and b are non-zero
        /// 
        /// There are 4 general cases:
        /// 1) c = a - b
        /// 2) c = a - (-b) = a + b
        /// 3) c = (-a) - b = (-a) + (-b)
        /// 4) c = (-a) - (-b) = b - a
        /// special: c = a - b = -(b - a)
        /// </summary>
        private static BigInteger SubtractPhase2( LinkedList<byte> lhs_digits, bool lhs_isneg, LinkedList<byte> rhs_digits, bool rhs_isneg )
        {
            //use algerbra to make sure a and b are positive
            if( !lhs_isneg && rhs_isneg ) // a - (-b) = a + b
            {
                return new BigInteger( AddPhase3( lhs_digits, rhs_digits ), false );
            }
            else if( lhs_isneg && !rhs_isneg ) // (-a) - b = (-a) + (-b) = -(a + b)
            {
                return new BigInteger( AddPhase3( lhs_digits, rhs_digits ), true ); //also: return (-a + b); works
            }
            else if( lhs_isneg && rhs_isneg ) // (-a) - (-b) = b - a
            {
                return SubtractPhase3( rhs_digits, lhs_digits );
            }
            else //a and b are positive.
            {
                return SubtractPhase3( lhs_digits, rhs_digits );
            }
        }

        /// <summary>
        /// a and b are non-zero and positive (returns zero if equal)
        /// </summary>
        private static BigInteger SubtractPhase3( LinkedList<byte> lhs_digits, LinkedList<byte> rhs_digits )
        {
            //we do a special check for a == b which means a - b = 0 for performance increases
            if( lhs_digits.Count == rhs_digits.Count ) //a is either equal to or less than b
            {
                int diff = CompareFirstDiffDigit( lhs_digits, rhs_digits );
                if( diff == 0 ) //they are equal
                {
                    return BigInteger.Zero;
                }
                else if( diff == -1 ) //a < b
                {
                    goto neg_b_minus_a;
                }
                else //a > b
                {
                    goto pos_a_minus_b;
                }
            }
            else if( lhs_digits.Count < rhs_digits.Count ) //a < b
            {
                goto neg_b_minus_a;
            }
            else //a > b
            {
                goto pos_a_minus_b;
            }

            //the following are the only two uses of SubtractPhase4, so we are ok so far in placing equality condition here, in Phase3
            neg_b_minus_a:
            return new BigInteger( SubtractPhase4( rhs_digits, lhs_digits ), true ); //a - b = -(b - a), when a < b
            pos_a_minus_b:
            return new BigInteger( SubtractPhase4( lhs_digits, rhs_digits ), false );
        }

        /// <summary>
        /// a and b are non-zero, positive, a > b, a does not equl b (use phase 3)
        /// </summary>
        private static LinkedList<byte> SubtractPhase4( LinkedList<byte> lhs_digits, LinkedList<byte> rhs_digits )
        {
            //use a standard borrowing subtraction algorithm
            LinkedList<byte> diff_digits = new LinkedList<byte>();
            LinkedListNode<byte> cur_lhs = lhs_digits.Last;
            LinkedListNode<byte> cur_rhs = rhs_digits.Last;
            int borrow = 0;
            int rhs_topindex = rhs_digits.Count - 1;
            for( int i = 0;; )
            {
                byte val_lhs = cur_lhs.Value;
                byte val_rhs = i <= rhs_topindex ? cur_rhs.Value : (byte)0;
                int diff = ( val_lhs - val_rhs - borrow );
                borrow = 0;
                if( diff < 0 )
                {
                    diff += 10;
                    borrow = 1;
                }
                diff_digits.AddFirst( (byte)diff );

                if( cur_lhs == lhs_digits.First )
                {
                    break;
                }

                cur_lhs = cur_lhs.Previous;
                if( ++i <= rhs_topindex )
                {
                    cur_rhs = cur_rhs.Previous;
                }
            }

            while( diff_digits.First.Value == (byte)0 )
            {
                diff_digits.RemoveFirst();
            }

            return diff_digits;
        }

        public static BigInteger operator *( BigInteger lhs, BigInteger rhs )
        {
            return Multiply( lhs, rhs );
        }

        private static LinkedList<byte> GenMultPart( int leading_mult, int zero_count )
        {
            LinkedList<byte> llist = new LinkedList<byte>();

            string leading_mult_str = leading_mult.ToString();
            for( int i = 0; i < leading_mult_str.Length; i++ )
            {
                llist.AddLast( byte.Parse( leading_mult_str[i].ToString() ) );
            }

            for( int j = 0; j < zero_count; j++ )
            {
                llist.AddLast( (byte)0 );
            }

            return llist;
        }

        /// <summary>
        /// Multiply two BigInts
        /// </summary>
        public static BigInteger Multiply( BigInteger lhs, BigInteger rhs )
        {
            if( lhs.digits == null || rhs.digits == null ) //0 * b = 0 = a * 0
            {
                return BigInteger.Zero;
            }

            LinkedList<byte> result_mult = null;

            int lzeros = lhs.digits.Count - 1;
            LinkedListNode<byte> ldigit = lhs.digits.First;
            for( ; lzeros >= 0; lzeros--, ldigit = ldigit.Next ) //standard multiplication algorithm
            {
                int rzeros = rhs.digits.Count - 1;
                LinkedListNode<byte> rdigit = rhs.digits.First;
                for( ; rzeros >= 0; rzeros--, rdigit = rdigit.Next )
                {
                    int leading_mult = ( ldigit.Value * rdigit.Value );
                    if( leading_mult != 0 ) //skip adding zero products
                    {
                        if( result_mult == null ) //init result_mult... don't like doing this check after already initialized
                        {
                            result_mult = GenMultPart( leading_mult, rzeros + lzeros );
                        }
                        else //add to result mult
                        {
                            AddTo( result_mult, GenMultPart( leading_mult, rzeros + lzeros ) ); //mutational addition for better memory management (less allocation of large linkedlists).
                        }
                    }
                }
            }

            return new BigInteger( result_mult, lhs.isneg ^ rhs.isneg );
        }

        /// <summary>
        /// Increment
        /// </summary>
        public static BigInteger operator ++( BigInteger bi )
        {
            return bi + BigInteger.One;
        }

        /// <summary>
        /// Increment
        /// </summary>
        public static BigInteger Increment( BigInteger bi )
        {
            return bi + BigInteger.One;
        }

        /// <summary>
        /// Decrement
        /// </summary>
        public static BigInteger operator --( BigInteger bi )
        {
            return bi - BigInteger.One;
        }

        /// <summary>
        /// Decrement
        /// </summary>
        public static BigInteger Decrement( BigInteger bi )
        {
            return bi - BigInteger.One;
        }

        ///// <summary>
        ///// version 1, very slow
        ///// Raise a BigInteger to an int power
        ///// </summary>
        //public static BigInteger Pow(BigInteger lhs, int power)
        //{
        //    if (power < 0)
        //        throw new ArgumentOutOfRangeException("rhs must be postive");
        //    else if (power == 0)
        //        return BigInteger.One;
        //    else if (power == 1)
        //        return lhs;

        //    BigInteger result = lhs; //Multiply is non-mutational, so don't need deep copy.
        //    for (int i = 1; i < power; i++)
        //        result = BigInteger.Multiply(result, lhs);

        //    return result;
        //}

        /// <summary>
        /// Raises a BigInteger to an uint power
        /// </summary>
        /// <param name="x"></param>
        /// <param name="n"></param>
        /// <see cref="http://en.wikipedia.org/wiki/Exponentiation_by_squaring"/>
        /// <returns></returns>
        public static BigInteger Pow( BigInteger x, uint n )
        {
            BigInteger result = BigInteger.One;
            while( n != 0 )
            {
                if( ( n & 1 ) != 0 ) /* n is odd, bitwise test */
                {
                    result = BigInteger.Multiply( result, x );
                }

                x = BigInteger.Multiply( x, x ); //might consider mutational mult. (would have to remember to reassign x to a deep copy of itself first)
                n /= 2; /* integer division, rounds down */
            }
            return result;
        }

        /// <summary>
        /// Perform division with remainder
        /// </summary>
        /// <param name="a">dividend</param>
        /// <param name="d">divisor</param>
        /// <param name="r">remainder</param>
        /// <returns>quotient</returns>
        public static BigInteger Divide( BigInteger a, BigInteger d, out BigInteger r )
        {
            if( d.digits == null ) // a / 0 DNE
            {
                throw new DivideByZeroException();
            }

            if( a.digits == null ) // 0 / d = 0, remainder 0
            {
                r = BigInteger.Zero;
                return BigInteger.Zero;
            }

            //now we know a.digits and d.digits are both non-null
            if( IsLessThan( a.digits, d.digits ) ) // a < d -> a / d = 0, remainder = a
            {
                r = a;
                return BigInteger.Zero;
            }

            LinkedList<byte> q = new LinkedList<byte>(); //we know q isn't zero
            LinkedList<byte> r_digits = null;
            for( LinkedListNode<byte> curbyte = a.digits.First; curbyte != null; curbyte = curbyte.Next )
            {
                if( r_digits != null || curbyte.Value != 0 ) //skip leading zeros
                {
                    if( r_digits == null )
                    {
                        r_digits = new LinkedList<byte>();
                    }

                    r_digits.AddLast( curbyte.Value );
                }

                //d.digits is never null, so if r_digits is null, it is "less" than d.digits
                if( r_digits == null || IsLessThan( r_digits, d.digits ) )
                {
                    if( q.Count != 0 ) //skip leading zeros
                    {
                        q.AddLast( (byte)0 );
                    }
                }
                else
                {
                    byte q_digit = BruteDivide( ref r_digits, d.digits ); //r_digits is both input and output parameter
                    if( q.Count != 0 || q_digit != 0 ) //skip leading zeros
                    {
                        q.AddLast( q_digit );
                    }
                }
            }

            r = new BigInteger( r_digits, a.isneg ); //even though mathematicians would always have r positive, we are being consistant with int type.
            return new BigInteger( q, a.isneg ^ d.isneg );
        }

        /// <summary>
        /// brut divisions where
        /// a_digits != null
        /// d_digits != null
        /// a and d are positive
        /// a is not less than d
        /// and quotient is a less than 10
        /// </summary>
        private static byte BruteDivide( ref LinkedList<byte> a_digits, LinkedList<byte> d_digits )
        {
            BigInteger r = new BigInteger( a_digits, false );
            int q = 0;
            for( ; r.digits != null; q++ )
            {
                r = SubtractPhase3( r.digits, d_digits );
                if( r.isneg )
                {
                    break;
                }
                else if( r.digits == null )
                {
                    q++;
                    break;
                }
            }

            if( r.isneg )
            {
                r = SubtractPhase3( d_digits, r.digits ); //d + r = d - abs(r), when r<0
            }

            a_digits = r.digits;
            return (byte)q;
        }

        /// <summary>
        /// Division operator
        /// </summary>
        public static BigInteger operator /( BigInteger lhs, BigInteger rhs )
        {
            return Divide( lhs, rhs );
        }

        /// <summary>
        /// Perform division without remainder
        /// </summary>
        public static BigInteger Divide( BigInteger lhs, BigInteger rhs )
        {
            BigInteger r;
            return Divide( lhs, rhs, out r );
        }

        /// <summary>
        /// The ceiling after division
        /// </summary>
        public static BigInteger DivideCeiling( BigInteger lhs, BigInteger rhs )
        {
            BigInteger r;
            BigInteger result = Divide( lhs, rhs, out r );

            if( r == BigInteger.Zero ) //no remainder
            {
                return result;
            }
            else if( lhs.isneg ^ rhs.isneg ) //result (with remainder) is negative
            {
                return result;
            }
            else
            {
                return ++result;
            }
        }

        /// <summary>
        /// The floor after division
        /// </summary>
        public static BigInteger DivideFloor( BigInteger lhs, BigInteger rhs )
        {
            BigInteger r;
            BigInteger result = Divide( lhs, rhs, out r );

            if( r == BigInteger.Zero ) //no remainder
            {
                return result;
            }
            else if( lhs.isneg ^ rhs.isneg ) //result (with remainder) is negative
            {
                return --result;
            }
            else
            {
                return result;
            }
        }

        /// <summary>
        /// mod: remainder after division, operator
        /// </summary>
        public static BigInteger operator %( BigInteger lhs, BigInteger rhs )
        {
            return Mod( lhs, rhs );
        }

        /// <summary>
        /// The remainder after division
        /// </summary>
        public static BigInteger Mod( BigInteger lhs, BigInteger rhs )
        {
            BigInteger r;
            Divide( lhs, rhs, out r );
            return r;
        }

        /// <summary>
        /// negation operator
        /// </summary>
        public static BigInteger operator -( BigInteger bi )
        {
            return Negate( bi );
        }

        /// <summary>
        /// The negation of a BigInteger
        /// </summary>
        public static BigInteger Negate( BigInteger bi )
        {
            return new BigInteger( bi.digits, !bi.isneg );
        }

        /// <summary>
        /// The absolute value of a BigInteger
        /// </summary>
        public static BigInteger Abs( BigInteger bi )
        {
            return new BigInteger( bi.digits, false );
        }
        #endregion Arithmetic Algorithms and Operators

        #region  Implicit Operators (from integral types to BigInteger)
        //note: implicit operators also serve as explicit operators

        public static implicit operator BigInteger( Byte value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        public static implicit operator BigInteger( SByte value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        public static implicit operator BigInteger( UInt16 value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        public static implicit operator BigInteger( Int16 value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        public static implicit operator BigInteger( UInt32 value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        public static implicit operator BigInteger( Int32 value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        public static implicit operator BigInteger( UInt64 value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        public static implicit operator BigInteger( Int64 value )
        {
            return BigInteger.Parse( value.ToString() );
        }

        //may consider implicit conversion from bool or datetime to since no precision / information loss
        //on the other hand, these are not integral types, so not purely logical
        #endregion Implicit Operators (from integral types to BigInteger)

        #region Explicit Operators (from boolean, DateTime, string, and rational types to BigInteger)
        public static explicit operator BigInteger( Decimal value )
        {
            return BigInteger.Parse( Math.Truncate( value ).ToString() );
        }

        public static explicit operator BigInteger( Double value )
        {
            return BigInteger.Parse( Math.Truncate( value ).ToString() );
        }

        public static explicit operator BigInteger( Single value )
        {
            return BigInteger.Parse( Math.Truncate( value ).ToString() );
        }

        public static explicit operator BigInteger( Boolean value )
        {
            return value ? BigInteger.One : BigInteger.Zero;
        }

        public static explicit operator BigInteger( DateTime value )
        {
            return value.Ticks;
        }

        //while a little unnatural and can always use BigInteger.Parse(string),
        //will serve well for purposes of Enumerable<string>.Cast<BigInteger>()
        public static explicit operator BigInteger( string value )
        {
            return BigInteger.Parse( value );
        }
        #endregion Explicit Operators (from boolean, DateTime, and rational types to BigInteger)

        #region Explicit Operators (from BigInteger to other types)
        public static explicit operator Byte( BigInteger value )
        {
            return Byte.Parse( value.ToString() );
        }

        public static explicit operator SByte( BigInteger value )
        {
            return SByte.Parse( value.ToString() );
        }

        public static explicit operator UInt16( BigInteger value )
        {
            return UInt16.Parse( value.ToString() );
        }

        public static explicit operator Int16( BigInteger value )
        {
            return Int16.Parse( value.ToString() );
        }

        public static explicit operator UInt32( BigInteger value )
        {
            return UInt32.Parse( value.ToString() );
        }

        public static explicit operator Int32( BigInteger value )
        {
            return Int32.Parse( value.ToString() );
        }

        public static explicit operator UInt64( BigInteger value )
        {
            return UInt64.Parse( value.ToString() );
        }

        public static explicit operator Int64( BigInteger value )
        {
            return Int64.Parse( value.ToString() );
        }

        //though will never throw, unnatural for implicit conversion
        public static explicit operator Boolean( BigInteger value )
        {
            return value.digits == null ? false : true;
        }

        //though will never throw, unnatural for implicit conversion
        public static explicit operator DateTime( BigInteger value )
        {
            return new DateTime( Int64.Parse( value.ToString() ) );
        }

        public static explicit operator Char( BigInteger value )
        {
            return (Char)UInt16.Parse( value.ToString() );
        }
        #endregion Explicit Operators (from BigInteger to other types)

        #region IConvertable
        //we risk redundant implementations for the sake of performance

        /// <summary>
        /// Returns the System.TypeCode for the Swensen.Object
        /// </summary>
        /// <returns></returns>
        public TypeCode GetTypeCode()
        {
            return TypeCode.Object;
        }

        bool IConvertible.ToBoolean( IFormatProvider provider )
        {
            return this.digits == null ? false : true;
        }

        byte IConvertible.ToByte( IFormatProvider provider )
        {
            return Byte.Parse( this.ToString() );
        }

        char IConvertible.ToChar( IFormatProvider provider )
        {
            return (Char)UInt16.Parse( this.ToString() );
        }

        DateTime IConvertible.ToDateTime( IFormatProvider provider )
        {
            return new DateTime( long.Parse( this.ToString() ) );
        }

        decimal IConvertible.ToDecimal( IFormatProvider provider )
        {
            return Decimal.Parse( this.ToString() );
        }

        double IConvertible.ToDouble( IFormatProvider provider )
        {
            return Double.Parse( this.ToString() );
        }

        short IConvertible.ToInt16( IFormatProvider provider )
        {
            return Int16.Parse( this.ToString() );
        }

        int IConvertible.ToInt32( IFormatProvider provider )
        {
            return Int32.Parse( this.ToString() );
        }

        long IConvertible.ToInt64( IFormatProvider provider )
        {
            return Int64.Parse( this.ToString() );
        }

        sbyte IConvertible.ToSByte( IFormatProvider provider )
        {
            return SByte.Parse( this.ToString() );
        }

        float IConvertible.ToSingle( IFormatProvider provider )
        {
            return Single.Parse( this.ToString() );
        }

        string IConvertible.ToString( IFormatProvider provider )
        {
            return this.ToString();
        }

        object IConvertible.ToType( Type conversionType, IFormatProvider provider )
        {
            return Convert.ChangeType( this, conversionType, provider ); //huh?
        }

        ushort IConvertible.ToUInt16( IFormatProvider provider )
        {
            return UInt16.Parse( this.ToString() );
        }

        uint IConvertible.ToUInt32( IFormatProvider provider )
        {
            return UInt32.Parse( this.ToString() );
        }

        ulong IConvertible.ToUInt64( IFormatProvider provider )
        {
            return UInt64.Parse( this.ToString() );
        }
        #endregion IConvertable

        #region Parse Methods
        /// <summary>
        /// Converts the string representation of an integer to its BigInteger equivalent
        /// </summary>
        public static BigInteger Parse( string value, uint radix = 10U )
        {
            if( value == null )
            {
                throw new ArgumentNullException( "value" );
            }

            BigInteger result = Zero;
            if( radix == 10 )
            {
                if( !TryParse( value, out result ) )
                {
                    throw new FormatException( "Input string was not in the correct format" );
                }
            }
            else if( value.Length > 0 )
            {
                value = value.ToUpperInvariant();

                BigInteger multiplier = One;

                bool isneg = value[0] == '-';
                int limit = isneg ? 1 : 0;
                for( int i = value.Length - 1; i >= limit; i-- )
                {
                    int posVal = (int)value[i];

                    if( posVal >= '0' && posVal <= '9' )
                        posVal -= '0';
                    else if( posVal >= 'A' && posVal <= 'Z' )
                        posVal = ( posVal - 'A' ) + 10;
                    else
                        throw new ArithmeticException( "Invalid string in constructor." );

                    if( posVal >= radix )
                        throw new ArithmeticException( "Invalid string in constructor." );

                    if( isneg )
                        posVal = -posVal;

                    result = result + ( multiplier * posVal );

                    if( ( i - 1 ) >= limit )
                        multiplier = multiplier * radix;
                }
            }

            return result;
        }

        /// <summary>
        /// Converts the string representation of an integer to its BigInteger equivalent.  
        /// A return value indicates whether the conversion succeeded
        /// </summary>
        public static bool TryParse( string numstr, out BigInteger result )
        {
            LinkedList<byte> digits;
            bool isneg;

            bool success = TryParse( numstr, out digits, out isneg );
            result = new BigInteger( digits, isneg ); //digits is null if success is false; thus result == BigInteger.zero

            return success;
        }

        /// <summary>
        /// Convert numstr into digits, and isneg BigInteger components; return success; digits is null if success is false
        /// </summary>
        private static bool TryParse( string numstr, out LinkedList<byte> digits, out bool isneg )
        {
            digits = null;
            isneg = false;

            if( numstr == null )
            {
                return false;
            }

            numstr = numstr.Trim();
            if( numstr.Length == 0 )
            {
                return false;
            }

            if( numstr[0] == '-' )
            {
                if( numstr.Length == 1 ) //"-"
                {
                    return false;
                }

                isneg = true;
            }

            int i = isneg ? 1 : 0;
            //skip leading zeros, including sole zero
            for( ; i < numstr.Length; i++ )
            {
                if( numstr[i] != '0' )
                {
                    break;
                }
            }

            if( i == numstr.Length ) //"-00000" == "0"
            {
                return true;
            }

            LinkedList<byte> digits_try = new LinkedList<byte>();
            for( ; i < numstr.Length; i++ )
            {
                byte digit;
                if( !byte.TryParse( numstr[i].ToString(), out digit ) )
                {
                    return false; //digits is still null
                }

                digits_try.AddLast( digit );
            }

            digits = digits_try;
            return true;
        }
        #endregion Parse Methods

        #region Eval
        //note that ops of longer length are listed first where using compound symbols (compared to BinaryOpTokens too)
        private static readonly string[] UnaryOpTokens =
            new string[]
                {
                    "gethashcode", "properdivisors", "divisors",
                    "negate", "abs", "++", "--",
                    "sum", "min", "max", "range",
                    "lcm", "gcd", "sqrt", "pow"
                };

        //note that ops of longer length are listed first where using compound symbols
        private static readonly string[] BinaryOpTokens =
            new string[]
                {
                    "==", "!=", ">=", "<=", ">=", "<=", ">", "<",
                    "/floor", "/ceiling", "/%", "/", "%", // "/%" is division with remainder
                    "+", "*", "^", "-" //make sure "-" is last
                };

        private static readonly string[] AllOpTokens = Enumerable.Union( UnaryOpTokens, BinaryOpTokens ).ToArray();

        /// <summary>
        /// Evaluates simple BigInteger expressions.
        /// </summary>
        public static object Eval( string expression )
        {
            if( expression == null )
            {
                throw new ArgumentNullException( "expression" );
            }

            string[] tokens = null;

            expression = expression.Replace( " ", string.Empty );

            string op = null;
            for( int i = 0; i < AllOpTokens.Length; i++ )
            {
                op = AllOpTokens[i];
                if( expression.Contains( op ) )
                {
                    tokens = expression.Split( new string[] {op}, StringSplitOptions.RemoveEmptyEntries );
                    break;
                }
            }

            if( tokens == null )
            {
                throw new FormatException( "no op tokens in expression" );
            }
            else if( tokens.Length == 2 )
            {
                BigInteger a = BigInteger.Parse( tokens[0] );
                BigInteger b = BigInteger.Parse( tokens[1] );

                switch( op )
                {
                    case "+":
                        return ( a + b );
                    case "-":
                        return ( a - b );
                    case "*":
                        return ( a * b );
                    case "^":
                        return ( BigInteger.Pow( a, Convert.ToUInt32( tokens[1] ) ) );
                    case "/%":
                        BigInteger r;
                        BigInteger q = Divide( a, b, out r );
                        return string.Format( "q = {0}, r = {1}", q, r );
                    case "/floor":
                        return DivideFloor( a, b );
                    case "/ceiling":
                        return DivideCeiling( a, b );
                    case "/":
                        return ( a / b );
                    case "%":
                        return ( a % b );
                    case "==":
                        return ( a == b );
                    case ">":
                        return ( a > b );
                    case ">=":
                        return ( a >= b );
                    case "<":
                        return ( a < b );
                    case "<=":
                        return ( a <= b );
                    default:
                        throw new FormatException( "Invalid binary operator" );
                }
            }
            else if( tokens.Length == 1 )
            {
                string[] strArgs = tokens[0].Split( ',' );
                BigInteger[] args = Parse( strArgs ).ToArray();
                BigInteger a = args[0];
                switch( op )
                {
                    case "gethashcode":
                        return a.GetHashCode();
                    case "abs":
                        return BigInteger.Abs( a );
                    case "negate":
                        return BigInteger.Negate( a );
                    case "properdivisors":
                        return ToString( a.ProperDivisors );
                    case "divisors":
                        return ToString( a.Divisors );
                    case "sum":
                        return Sum( args );
                    case "range":
                        return ToString( BigInteger.Range( args[0], args[1] ) );
                    case "min":
                        return Min( args );
                    case "max":
                        return Max( args );
                    case "lcm":
                        return Lcm( args[0], args[1] );
                    case "gcd":
                        return Gcd( args[0], args[1] );
                    case "pow":
                        return Pow( a, Convert.ToUInt32( strArgs[1] ) );
                    case "sqrt":
                        return Sqrt( a );
                    case "++":
                        return ( ++a );
                    case "--":
                        return ( --a );
                    default:
                        throw new FormatException( "Invalid unary operator" );
                }
            }

            throw new FormatException( "invalid expression" );
        }

        private static IEnumerable<BigInteger> Parse( IEnumerable<string> input )
        {
            foreach( string i in input )
            {
                yield return BigInteger.Parse( i );
            }
        }

        private static string ToString( object obj )
        {
            return obj == null ? string.Empty : obj.ToString();
        }

        private static string ToString<T>( IEnumerable<T> objs )
        {
            if( objs == null )
            {
                return string.Empty;
            }

            return string.Join( ", ", objs.Select<T, string>( ( T x ) => x.ToString() ).ToArray() );
        }
        #endregion Eval

        #region Common Algorithms
        /// <summary>
        /// Yields the inclusive range of values from start to end
        /// </summary>
        public static IEnumerable<BigInteger> Range( BigInteger start, BigInteger end )
        {
            //if (upperBound < lowerBound)
            //throw new ArgumentOutOfRangeException("upperBound cannot be less than lowerBound");

            if( end >= start )
            {
                do
                {
                    yield return start;
                    start++;
                } while( start <= end );
            }
            else
            {
                do
                {
                    yield return start;
                    start--;
                } while( start >= end );
            }
        }

        /// <summary>
        /// Calculates the maximum of two BigInts
        /// </summary>
        public static BigInteger Max( BigInteger lhs, BigInteger rhs )
        {
            return lhs >= rhs ? lhs : rhs;
        }

        /// <summary>
        /// Calculates the maximum of a set of BigInts
        /// </summary>
        public static BigInteger Max( params BigInteger[] seq )
        {
            return seq.Max();
        }

        /// <summary>
        /// Calculates the maximum of a set of BigInts
        /// </summary>
        public static BigInteger Max( IEnumerable<BigInteger> seq )
        {
            return seq.Max();
        }

        /// <summary>
        /// Calculates the minimum of two BigInts
        /// </summary>
        public static BigInteger Min( BigInteger lhs, BigInteger rhs )
        {
            return lhs <= rhs ? lhs : rhs;
        }

        /// <summary>
        /// Calculates the minimum of a set of BigInts
        /// </summary>
        public static BigInteger Min( params BigInteger[] seq )
        {
            return seq.Min();
        }

        /// <summary>
        /// Calculates the minimum of a set of BigInts
        /// </summary>
        public static BigInteger Min( IEnumerable<BigInteger> seq )
        {
            return seq.Min();
        }

        /// <summary>
        /// Calculates the sumation of a set of BigInts
        /// </summary>
        public static BigInteger Sum( params BigInteger[] seq )
        {
            return Sum( (IEnumerable<BigInteger>)seq );
        }

        /// <summary>
        /// Calculates the sumation of a set of BigInts
        /// </summary>
        public static BigInteger Sum( IEnumerable<BigInteger> seq )
        {
            bool seeded = false;
            BigInteger sum = BigInteger.Zero;
            foreach( BigInteger bi in seq )
            {
                if( bi != BigInteger.Zero ) //don't bother
                {
                    if( !seeded ) //seed sum
                    {
                        sum = new BigInteger( new LinkedList<byte>( bi.digits ), bi.isneg ); //deep copy for AddTo
                        seeded = true;
                    }
                    else if( sum.isneg ^ bi.isneg ) //can use AddTo if and only if sum and bi are both positive or both negative
                    {
                        sum += bi;
                    }
                    else //when we can
                    {
                        BigInteger.AddTo( sum.digits, bi.digits );
                    }
                }
            }

            return sum;
        }

        /// <summary>
        /// Finds the greatest common divisor of two BigInts
        /// </summary>
        //public static BigInteger Gcd(BigInteger lhs, BigInteger rhs)
        //{
        //    if (lhs == BigInteger.Zero)
        //        throw new ArgumentOutOfRangeException("lhs", "Gcd is not defined for BigInteger.Zero");
        //    else if (lhs == BigInteger.Zero) //bug
        //        throw new ArgumentOutOfRangeException("rhs", "Gcd is not defined for BigInteger.Zero");
        //    lhs = BigInteger.Abs(lhs);
        //    rhs = BigInteger.Abs(rhs);
        //    return Max(Enumerable.Intersect(lhs.Divisors, rhs.Divisors)); //not fast, should use euclidean method
        //}
        /// <summary>
        /// Finds the greatest common divisor of two BigInts
        /// </summary>
        /// <param name="lhs"></param>
        /// <param name="rhs"></param>
        /// <see cref="http://en.wikipedia.org/wiki/Euclidean_algorithm"/>
        /// <returns></returns>
        public static BigInteger Gcd( BigInteger lhs, BigInteger rhs )
        {
            if( lhs.digits == null )
            {
                throw new ArgumentOutOfRangeException( "lhs", "Gcd is not defined for BigInteger.Zero" );
            }
            else if( rhs.digits == null )
            {
                throw new ArgumentOutOfRangeException( "rhs", "Gcd is not defined for BigInteger.Zero" );
            }

            while( rhs.digits != null )
            {
                if( lhs > rhs ) //not sure if we can trust digits not to be null
                {
                    lhs = BigInteger.Subtract( lhs, rhs );
                }
                else
                {
                    rhs = BigInteger.Subtract( rhs, lhs );
                }
            }

            return lhs;
        }

        //public static BigInteger GCD(params BigInteger[] seq)
        //{
        //    return GCD((IEnumerable<BigInteger>)seq);
        //}

        //public static BigInteger GCD(IEnumerable<BigInteger> seq)
        //{

        //}

        /// <summary>
        /// Finds the least common multiple of two BigInts
        /// </summary>
        public static BigInteger Lcm( BigInteger lhs, BigInteger rhs )
        {
            if( lhs.digits == null || rhs.digits == null )
            {
                return BigInteger.Zero;
            }

            return ( lhs * rhs ) / Gcd( lhs, rhs );
        }

        /// <summary>
        /// Calcultes the truncated square root of a BigInteger
        /// </summary>
        /// <param name="value">the operand</param>
        /// <see cref="http://www.codecodex.com/wiki/index.php?title=Calculate_an_integer_square_root"/>
        /// <returns>Square root of value</returns>
        public static BigInteger Sqrt( BigInteger value )
        {
            if( value.digits == null )
            {
                return BigInteger.Zero; // Avoid zero divide
            }

            BigInteger n = DivideCeiling( value, BigInteger.Two ); // Initial estimate, never low
            BigInteger n1 = ( n + ( value / n ) ) / BigInteger.Two;
            while( n1 < n )
            {
                n = n1;
                n1 = ( n + ( value / n ) ) / BigInteger.Two;
            } // end while

            return n;
        }

        // end Isqrt()
        #endregion Common Algorithms

        #region IXmlSerializable
        //default xml serialization is unsuitable (public properties and fields are exactly what we don't want!)

        void IXmlSerializable.WriteXml( XmlWriter writer )
        {
            writer.WriteString( this.ToString() );
        }

        void IXmlSerializable.ReadXml( XmlReader reader )
        {
            TryParse( reader.ReadString(), out this.digits, out this.isneg );
        }

        XmlSchema IXmlSerializable.GetSchema()
        {
            return ( null );
        }
        #endregion IXmlSerializable
    }
}